# procedimiento para aplicar una mascara con el objeto
# de definir el area util en las imagenes del CASPOL. 
# 
# debido al vigneteo producido por el Savart, la lamina
# y los filtros, la imagen util en el detector se reduce
# a un circulo fuera del cual la intensidad de la luz
# cae rapidamente a cero, lo que introduce mucho ruido 
# en los rincones y bordes de las imagenes dificultando
# el uso de los restantes script para procesar los datos.
# 
# Este script permite aplicar a una lista de imagenes la
# mascara creada por MKMASC y  reemplazar los pixeles externos
# que se descartan por un valor elegido por el usuario.
#
# los parametros de entrada son:
#
# ARCHIN      = lista de imagenes a procesar
# ARCHOUT     = lista de imagenes de salida
# VALUE       = valor para reemplazar en los bordes
# LVALUE      = lista con los valores para reemplazar en los bordes para cada imagen
# MASK        = mascara a aplicar
# FLIST       = flag que indica que se usa una lista para los valores de reemplazo
#
#
# Noviembre 2012
#

procedure mascara (archin, archout,mask)

string archin        {prompt="list of images to apply the mask"}
string archout       {prompt="list of output images"}
string mask          {prompt="mask name"}
real value=0.        {prompt="constant pixel value to use out of the mask"}
string lvalue        {prompt="list of constant pixel values to use out of the mask in each image"}
bool   flist=no      {prompt="do you use a list of constant pixels for each image?"}
 
struct *lista,*lista1,*lista2

begin
  string infile,outfile,imag,patron
  string car,arch,arch1,expr,valor
  string tfile,tfile1,tfile2
  string ifile
  int dx,dy
  real mod
      
# inicializa archivos y punteros

 infile=archin
 outfile=archout
 patron=mask
 valor=value

# se asegura que las imagenes a procesar estan en una lista

  tfile=mktemp("temp")
  car=substr(infile,1,1)
  if(car != "@"){
    files(infile,>>tfile)
  }else{
    if(access(tfile)) delete(tfile,ver-)
    copy(substr(infile,2,strlen(infile)),tfile)
  }

# se asegura que las imagenes de salida estan en una lista

  tfile1=mktemp("temp")
  car=substr(outfile,1,1)
  if(car != "@"){
    files(outfile,>>tfile1)
  }else{
    if(access(tfile1)) delete(tfile1,ver-)
    copy(substr(outfile,2,strlen(outfile)),tfile1)
  }

# se asegura que los valores de reemplazo esten en una lista

  if(flist == yes){
    tfile2=mktemp("temp")
    car=substr(valor,1,1)
    if(car != "@"){
      files(lvalor,>>tfile2)
    }else{
      if(access(tfile2)) delete(tfile2,ver-)
      copy(substr(lvalor,2,strlen(lvalor)),tfile2)
    }
  }

# procesa las imagenes

  lista=tfile
  lista1=tfile1
  if(flist == yes)lista2=tfile2

  while(fscan(lista,arch) != EOF){

    imag=mktemp("temp")
    imcopy(arch,imag,ver-)

# dimensiones de la imagen

    imgets(imag,"i_naxis1")
    dx=int(imgets.value)
    imgets(imag,"i_naxis2")
    dy=int(imgets.value)

# aplica mascara para seleccionar la zona util
 
    imarith(imag,"*",patron,imag)

# invierte mascara para reemplazar el borde por el
# valor solicitado

    ifile=mktemp("temp")
    expr="a < 1 ? 1 : 0"
    imexpr(expr,ifile,patron)
#    imexpr(expr,ifile,patron,dims=dx//","//dy)

    if(flist == yes){
      car=fscan(lista2,mod)
    }else{
      mod=value
    }
    imarith(ifile,"*",mod,ifile)

    imarith(imag,"+",ifile,imag)

# imagen de salida

    car=fscan(lista1,arch1)
    imcopy(imag,arch1,ver-)

    imdel(ifile,ver-)
    imdel(imag,ver-)

  }

# salida
  
  delete(tfile,ver-)
  delete(tfile1,ver-)
  if(flist == yes)delete(tfile2,ver-)
  lista=""
  lista1=""
  lista2=""

end
