# procedimiento para generar una mascara con el objeto
# de definir el area util en las imagenes del CASPOL. 
# 
# debido al vigneteo producido por el Savart, la lamina
# y los filtros, la imagen util en el detector se reduce
# a un circulo fuera del cual la intensidad de la luz
# cae rapidamente a cero, lo que introduce mucho ruido 
# en los rincones y bordes de las imagenes dificultando
# el uso de los restantes script para procesar los datos.
# 
# Este script permite definir una mascara circular para 
# poner a cero las zonas externas mas ruidosas.
#
# la imagen de entrada debe ser un flat (preferentemente) 
# y los parametros de entrada son:
#
# RAD        = radio del area circular a definir
# CX,CY      = centro aproximado de la mascara
# LADOX,LADOY= lados del area rectangular donde se busca el
#              centro de la mascara
#
# el script busca la mascara de las dimensiones solicitadas
# que maximiza el area efectiva de la imagen, con dimensiones
# igual a las de la imagen de entrada y pixels de tipo INT.
#
#
# Noviembre 2012
#

procedure mkmasc (archin, archout)
real   rad=450.  {min=1, prompt="radius of useful region in image"}
int    cx=512    {min=1, prompt="approx. mask center in X"}
int    cy=512    {min=1, prompt="approx. mask center in Y"}
int    ladox=10  {min=5, prompt="X side of the search area"}
int    ladoy=10  {min=5, prompt="Y side of the search area"}

string archin        {prompt="reference flat image"}
string archout       {prompt="output mask"}
 
struct *lista         

begin
  string imag,outfile,expr
  string tfile,tfile1,datos
  int dx,dy,rx,ry,cenx,ceny
  real mean,mean0
      
# inicializa archivos y punteros

 imag=archin
 outfile=archout
 if(access(outfile)) imdel(outfile,ver-)

# dimensiones de la imagen

  imgets(imag,"i_naxis1")
  dx=int(imgets.value)
  imgets(imag,"i_naxis2")
  dy=int(imgets.value)

# define mascara externa

  expr="sqrt((I-a)**2+(J-b)**2) <c ? 0 : 1"

# comienza busqueda 

  mean0=100000000.
  cenx=dx
  ceny=dy

  for(i=-ladox/2;i<=ladox/2;i=i+1){
    for(j=-ladoy/2;j<=ladoy/2;j=j+1){

# crea mascara

      tfile=mktemp("temp")
      rx=cx+i
      ry=cy+j
      imexpr(expr,tfile,rx,ry,rad,dims=dx//","//dy)

# aplica la mascara y calcula valor medio resultante

      tfile1=mktemp("temp")
      imarith(imag,"*",tfile,tfile)
      imstatistics(tfile,fields="mean",format-, > tfile1)
      lista=tfile1
      datos=fscan(lista,mean)
      print(rx,ry,mean,mean0)

# es un minimo? 

      if(mean < mean0){
        mean0=mean
        cenx=rx
        ceny=ry
      }

      delete(tfile//".fits",ver-)
      delete(tfile1,ver-)
      lista=""
    }
  }

# mascara final

  tfile=mktemp("temp")

  expr="sqrt((I-a)**2+(J-b)**2) <c ? 1 : 0"
  imexpr(expr,tfile,cenx,ceny,rad,dims=dx//","//dy)

  print("")
  print("Centro de la mascara: X=",cenx," Y=",ceny)

# salida
  
  imcopy(tfile,outfile,ver-)
  
  imdel(tfile,ver-)
end
