# procedimiento para re-ordenar el listado de estrellas
# encontrado por DAOFIND, RIMCURSOR, etc. de tal modo
# de poner juntas las correspondientes a la imagen 
# ordinaria y extraordinaria generadas por el Savart
# en el polarimetro CASPOL
#
# el archivo de entrada es el de salida de DAOFIND, RIMCURSOR, etc.,
# y los parametros de entrada son:
#
# SX,SY= distancia en (X,Y) entre las imagen ordinaria y extraordinaria
# DX,DY= error en (X,Y) permitido
#
# las distancias SX y SY se calculan restando las coordenadas del objeto
# mas a la izquierda del objeto mas a la derecha
# 
# el archivo de salida solo guarda las coordenadas (X,Y), y el codigo 
# de identificacion de la estrella
#
# el script puede procesar hasta 1000 estrellas (2 imagenes para cada una)
#
# Noviembre 2012
#

procedure ordenar (archin, archout)
real   sx=38.387  {min=0, prompt="x-axis offset"}
real   sy=00.000  {prompt="y-axis offset"}
real   dx=2.      {min=1., prompt="x-axis error"}
real   dy=2.      {min=1., prompt="y-axis error"}

string archin        {prompt="coordinate file"}
string archout       {prompt="output file"}
 
struct *lista         

begin
  string tempfile, outfile
  struct linea, linea1
  int ii,jj,ifin,id,num
  int id0[2000], flg[2000]
  real sx0,sy0, sxup, sxdw, syup, sydw
  real cx,cy,mg,v1,v2,v3,ddx,ddy
  real cx0[2000], cy0[2000]

      
# inicializa archivos y punteros

 lista=archin
 outfile=archout
 if(access(outfile)) delete(outfile,ver-)

 sy0=sy
 sx0=sx
 
# limites en X e Y

 sxup=sx0+dx
 sxdw=sx0-dx
 syup=sy0+dy
 sydw=sy0-dy
     
# comienza lectura del archivo de entrada
  
  ii=0
  while(fscan(lista,linea) != EOF){

# no es un comentario

    if(substr(linea,1,1) != "#"){
      ii=ii+1
      linea1=fscan(linea,cx,cy)
      cx0[ii]=cx
      cy0[ii]=cy
      id0[ii]=ii
      flg[ii]=0
    }
  } 

  ifin=ii
  num=0

# busca los pares

  tempfile = mktemp("./temp")
  print("# XCEN     YCEN  ID", >> tempfile)

  for(ii=1;ii<=ifin;ii+=1){

# no fue seleccionado antes

    if(flg[ii] == 0){
      for(jj=ii+1;jj<=ifin;jj+=1){

# no fue seleccionado antes

        if(flg[jj] == 0){
          ddx=cx0[jj]-cx0[ii]
          ddy=cy0[jj]-cy0[ii]
          if((ddx < sxup) && (ddx > sxdw)){
            if((ddy < syup) && (ddy > sydw)){

# encontro un par

              print(cx0[ii],"  ",cy0[ii],"  ",id0[ii],>>tempfile)
              print(cx0[jj],"  ",cy0[jj],"  ",id0[jj],>>tempfile)
              print("Pair: ",id0[ii]," ",id0[jj])
              flg[ii]=1
              flg[jj]=1
              num=num+1
            }
          }
        }
      }
    }
  }

# salida

  print("Total pairs: ",num)
  print("Total stars: ",ifin)
  print(" ")
  print("Lonely stars:")
  for(ii=1;ii<=ifin;ii+=1){
    if(flg[ii] == 0)
      print(cx0[ii]," ",cy0[ii]," ",id0[ii])
  }
  print(" ")

  copy(tempfile,outfile)
  delete(tempfile,ver-)
  lista=""

end
