# Script para analizar los resultados obtenidos con POLRED.
#
# Se ingresa con los dos archivos de salida de POLRED 
# (POLRED.OUT y POLRED.CUR) que son leidos y graficados de
# diferentes formas. Se puede obtener graficos de:
#
# - polarizacion vs apertura.
# - error de polarizacion vs apertura.
# - error relativo de polarizacion vs apertura.
# - angulo de posicion vs apertura.
# - Q vs apertura.
# - U vs apertura.
# - Q vs U.
# - intensidad vs posicion de la lamina.
#
# Para ejecutar el script se requiere el paquete PLOT.
#
#
# Noviembre 2013
#

procedure polana (archpol,archcur)

string archpol        {prompt="polarimetry result"}
string archcur        {prompt="Intensity result"}

struct *lista

begin
 struct line
 string arch1,arch2
 string st1,st2,st3
 string ejex,ejey,titulo
 string tfile,tgcur,tecla,press
 int res,ii,fln
 real v0,v1,v2,v3,v4,v5,v6,v7,v8,v9,vx,vy
 real lx0,lx1,ly0,ly1,app
      
# controla que este cargado el paquete PLOT

 if (! defpac ("plot")){
   print("The PLOT package must be loaded.")
   bye
 }

# inicializa archivos y punteros

 arch1=archpol
 arch2=archcur

 app=3.

# muestra diferentes graficos de los parametros
# polarimetricos

 press="l"

# bucle de lectura de teclado

 while(press != "x"){

# grafico de intensidad

   if(press == "i"){
     print(" ")
     print("Which aperture?: ")
     res=scan(app)
   }
# despliega parametros polarimetricos en pantalla

   if(press == "l"){
     lista=arch1
     while(fscan(lista,line) != EOF){ 
       tecla = substr(line,1,1)
       if(tecla != "#")print(line)
     }
     lista=""
     press= "h"
   }

# despliega el help

   if(press == "h"){
     print(" ")
     print("h - This help.")
     print("l - Show the polarimetric results.")
     print("p - Percent of pol. light vs Aperture.")
     print("t - Position angle vs Aperture.")
     print("e - Error in percent of pol. light vs Aperture.")
     print("s - Relative error of pol. light vs Aperture.")
     print("q - Q vs Aperture.")
     print("u - U vs Aperture.")
     print("c - Q vs U.")
     print("i - Intensity vs. Half-wave position.")
     print("x - Exit.")
     print(" ")
     press="p"
   }

# se prepara para mostrar un grafico

   else{
     lista=arch1
     tfile = mktemp("./temp")
     lx0=100000.
     lx1=-100000.
     ly0=10000.
     ly1=-10000.
     vx=0.
     vy=0.

# lee archivo de entrada

     while(fscan(lista,line) != EOF){ 
   
       tecla=substr(line,4,4)
       if(tecla == "."){
         res=fscan(line,v0,v1,v2,v3,v4,v5,v6,v7,v8,v9)

# si el grafico es de intensidad, guarda para la apertura
# pedida el modulo del vector y su angulo de posicion

         if((press == "i") && (v0 == app)){
           vx=v5
           vy=v7
         }

# grafico de polarizacion vs apertura

         else if(press == "p"){
           vx=v0
           vy=v5
           ejex="Aperture (px)"
           ejey="Pol (%)"
           titulo="Percent of polarized light vs. Aperture"
         }

# grafico de error vs apertura

         else if(press == "e"){
           vx=v0
           vy=v6
           ejex="Aperture (px)"
           ejey="Error in Pol (%)"
           titulo="Error in percent of polarized light vs. Aperture"
         }

# grafico de error relativo vs apertura

         else if(press == "s"){
           vx=v0
           vy=v6/v5
           ejex="Aperture (px)"
           ejey="Rel. error in Pol (%)"
           titulo="Relative error of polarized light vs. Aperture"
         }

# grafico de ang. de posicion vs apertura

         else if(press == "t"){
           vx=v0
           vy=v7
           ejex="Aperture (px)"
           ejey="Pos. angle (deg.)"
           titulo="Position angle vs. Aperture"
         }

# grafico de Q vs apertura

         else if(press == "q"){
           vx=v0
           vy=v1
           ejex="Aperture (px)"
           ejey="Q (%)"
           titulo="Q vs. Aperture"
         }

# grafico de U vs apertura

         else if(press == "u"){
           vx=v0
           vy=v3
           ejex="Aperture (px)"
           ejey="U (%)"
           titulo="U vs. Aperture"
         }

# grafico de Q vs U

         else if(press == "c"){
           vx=v1
           vy=v3
           ejex="Q (%)"
           ejey="U (%)"
           titulo="Q vs. U"
         }

# determina minimos  maximos para el grafico pedido

         print(vx,vy,>>tfile)
         if(vx < lx0)lx0=vx
         if(vx > lx1)lx1=vx
         if(vy < ly0)ly0=vy
         if(vy > ly1)ly1=vy
   
       }
     }
     lista=""

# grafico de intensidad para una apertura

     if((press == "i") && (vy != 0.)){
       lx0=100000.
       lx1=-100000.
       ly0=10000.
       ly1=-10000.
       lista=arch2
       tfile=mktemp("./temp")
       fln=0

# lee archivo de entrada para extraer intensidades
# en la apertura correcta

       while(fscan(lista,line) != EOF){
         res=fscan(line,st1,st2,st3)
         if((fln == 1) && (st1 != "#")){
           res=fscan(line,v0,v1,v2)
           print(v0,v1,>>tfile)
           if(v0 < lx0)lx0=v0
           if(v0 > lx1)lx1=v0
           if(v1 < ly0)ly0=v1
           if(v1 > ly1)ly1=v1
         }
         else{      
           if(st2 == "Aper:"){
             res=fscan(st3,v9)
             if(v9 == app)fln=1
             else fln=0
           }   
         }
       }

       lista=""

# minimos y maximos para el grafico

       if(lx0 < 180.)lx0=-10.
       else lx0=170.
       if(lx1 > 180.)lx1=370.
       else lx1=190.
       if(ly0 < 0.)ly0=ly0*1.2
       else ly0=ly0*0.8
       if(ly1 < 0.)ly1=ly1*0.8
       else ly1=ly1*1.2

# arma el grafico de intensidad

       ejex="Half-wave position (deg)"
       ejey="Intensity"
       titulo="Polarization curve for Aper. "//app
       graph(tfile,wx1=lx0,wx2=lx1,wy1=ly0,wy2=ly1,pointmode=yes,overplot=no,append=no,xlabel=ejex, ylabel=ejey,title=titulo)
   
       delete(tfile,ver-)

# calcula curva teorica

       vy=vy/180.*3.141592
       vx=vx/100.
       tfile=mktemp("./temp")
       for(ii=lx0;ii<=lx1;ii=ii+2){
         v0=vx*cos(2.*vy)*cos(4.*ii/180.*3.141592)+vx*sin(2.*vy)*sin(4.*ii/180.*3.141592)
         print(ii,v0,>>tfile)
       }

# sobre-escribe con la curva teorica

       graph(tfile,wx1=lx0,wx2=lx1,wy1=ly0,wy2=ly1,pointmode=no,color=5,overplot=yes,append=yes)
       delete(tfile,ver-)

     }

# graficos en funcion de la apertura

     else if(vx != 0.){

# minimos y maximos para el grafico

       if(lx0 < 0.)lx0=lx0*1.2
       else lx0=lx0*0.8
       if(lx1 < 0.)lx1=lx1*0.8
       else lx1=lx1*1.2
       if(ly0 < 0.)ly0=ly0*1.2
       else ly0=ly0*0.8
       if(ly1 < 0.)ly1=ly1*0.8
       else ly1=ly1*1.2

       graph(tfile,wx1=lx0,wx2=lx1,wy1=ly0,wy2=ly1,pointmode=yes,overplot=no,append=no,xlabel=ejex, ylabel=ejey,title=titulo)
   
       delete(tfile,ver-)
     }

# lee el cursor grafico para detectar la presion de
# alguna tecla

     tgcur = mktemp("./temp")
     print(gcur,>tgcur)
     lista=tgcur
     res=fscan(lista,v0,v1,v2,press)
     lista=""
     delete(tgcur,ver-)
   }
 }
# salida
end
