# procedimiento para correr el programa de reduccion 
# POLRED desde IRAF. 
# 
# este script crea el archivo de parametros, llama al
# ejecutable, y copia el archivo de salida del programa
# (POLRED.OUT) con el nombre indicado.
#
# los parametros de entrada son:
#
# RDNOISE y GAIN = readout noise y ganacia del detector
# NAPER          = numero de aperturas usadas para la fotometria
# QINST y UINST  = Q y U instrumental
# EQINST y EUINST= errores en Q y U instrumental
# APOS           = correccion al angulo de posicion
# NORM           = flag que indica si quiere renormalizacion o no
# EXE            = programa ejecutable
#
#
# Noviembre 2012
#

procedure polred (archin, archout)
real   rdnoise=7.40  {min=0., prompt="readout noise of the detector"}
real   gain=1.98     {min=0.5, prompt="gain of the detector"}
int    naper=4       {min=1,max=8,prompt="number of apertures used"}
real   qinst=0.      {prompt="Q component of the instrumental polarization"}
real   uinst=0.      {prompt="U component of the instrumental polarization"}
real   eqinst=0.     {prompt="error in Q component of the instrumental polarization"}
real   euinst=0.     {prompt="error in U component of the instrumental polarization"}
real   apos=0.       {prompt="correction of the position angle of polarization"}
bool   norma=yes      {prompt="do you want renormalization?"}
string exe="./polred" {prompt="polred executable file"}

string archin        {prompt="file with the observations"}
string archout       {prompt="output file"}
 
#struct *lista         

begin
  string infile,outfile,expr
  string tfile
      
# inicializa archivos

  infile=archin
  outfile=archout
  if(access(outfile)) delete(outfile,ver-)

# crea archivo de parametros

  tfile="polred.in"
  if(access(tfile)) delete(tfile,ver-)

  print("")
  print("Making parameter file...")
  print(rdnoise,gain, >> tfile)
  print(naper, >> tfile)
  print(qinst,uinst, >> tfile)
  print(eqinst,euinst, >> tfile)
  print(apos, >> tfile)
  if(norma == yes){
    print("1", >> tfile)
  }else{
    print("0", >> tfile)
  }
  print(infile, >> tfile)

# ejecutando programa de reduccion

  print("")
  print("Executing the reduction program...")

  if(access("polred.out")) delete("polred.out")

  print("")
  expr="!"//exe
  print(expr)|cl

  print("")
  if(access("polred.out")){
    print("Copying the output file...")
    copy("polred.out",outfile)
    delete("polred.out")
  }else{
    print("The output file was not created")
  }
  delete("polred.in")
  print("Done.")

# salida
  
end
