! programa para reducir las observaciones polarimetricas 
! obtenidas con CASPOL.
!
! El programa tiene dos archivos de entrada. El primero,
! denominando POLRED.IN, es un archivo con parametros de
! configuracion:
!
! - readout noise (e-) y ganancia del detector (e-/ADU)
! - numero de aperturas utilizadas
! - componentes Q y U de la pol. instrumental (%)
! - errores en Q y U instrumentales (%)
! - correccion al angulo de posicion (grados)
! - flag que indica si se desea renormalizacion (1-0)
! - nombre del archivo con las observaciones
!
! El segundo archivo de entrada es el que contiene las 
! observaciones y corresponde al archivo de salida del script
! POLEXT.CL de IRAF. En este archivo las mediciones de las
! estrellas siempre estan en el mismo orden en cada imagen
! y los valores ordinarios y extraordinarios para cada una 
! de ellas estan uno a continuacion del otro.
!
! El programa acepta 1000 estrellas, 16 posiciones de la
! lamina de media onda, y 8 aperturas.
!
! rgh - Noviembre 2012
!
! se modifica el calculo de q y u. Se hace una salida a
! archivo (POLRED.CUR) con los valores de phi(i), z(i) y
! sigma(zi) para las diferentes aperturas utilizadas con
! el objeto de poder adaptar una curva graficamente.
!
! rgh v1.2 - Noviembre 2013
!
!
program polred
implicit none
  integer,parameter ::d=selected_real_kind(15,200)
  integer,parameter :: nst=1000,nap=8
  character(30) :: arch,imag,imag0
  character(11) :: fecha0,hora0,fecha,hora
  character :: fil,fil0
  integer :: ii,jj,ij,tstar,fstar,id
  integer :: aper,ren,flg,ihw
  real(d),dimension(nst,16,nap) :: fxo,fxe,aro,are
  real(d),dimension(nst,16) :: fso,fse,aso,ase
  real(d),dimension(nap) :: rap
  real(d),dimension(2+3*nap) :: vv
  real(d),dimension(16) :: phi,vcos,vsin,vint
  real(d) :: rdnoise,gain,vq0,vu0,sq0,su0,th0
  real(d) :: pi,hw,sav,sav0
  real(d) :: tfxo,tfxe,tfx,tfs,arap,arsk,rdn2
  real(d) :: scl,snt,vcc,vss,vcs,vcz,vsz
  real(d) :: det,sq,su,vq,vu,pol,suma2
  real(d) :: sig,sigq,sigu,sigpol,th,sigth,sigz

  pi=4._d*atan(1._d)

  fxo=0._d
  fxe=0._d
  aro=0._d
  are=0._d
  fso=0._d
  fse=0._d
  aso=0._d
  ase=0._d
  phi=-1._d

! datos a pantalla
!
  write(*,*)'------------------------------------------'
  write(*,*)'POLRED v1.2 - Reduction program for CASPOL'
  write(*,*)'November 2013'
  write(*,*)'------------------------------------------'
  write(*,*) 
 
! lee el archivo de parametros
!
  open(7,file='polred.in')
  read(7,*)rdnoise,gain
  read(7,*)aper
  read(7,*)vq0,vu0
  read(7,*)sq0,su0
  read(7,*)th0
  read(7,*)ren
  read(7,*)arch
  close(7)

  vq0=vq0/100._d
  vu0=vu0/100._d
  sq0=sq0/100._d
  su0=su0/100._d

  if(aper > 8)then
    write(*,*)'El maximo de aperturas permitidas es 8.'
    stop
  endif

! datos a pantalla
!
  write(*,*)'Number of apertures: ',aper
  write(*,*)'Images and HW:'

! lee datos de las observaciones
!
  tstar=0
  fstar=0
  imag0=' '
  open(7,file=arch)
  do

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! lee datos de imagen ordinaria
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    read(7,*,iostat=flg)imag,id,(vv(jj),jj=1,2+3*aper),fil,hw,sav,fecha,hora
    if(flg /=0)exit

! si es la primera lectura guarda nombre de imagen,
! filtro y posicion del Savart
!
    if(imag0 == ' ')then
      imag0=imag

! datos a pantalla
!
      write(*,*)imag,' ',hw
      fecha0=fecha
      hora0=hora
      fil0=fil
      sav0=sav
    endif

! es un cambio de imagen
!
    if(imag0 /= imag)then
      if(fstar == 0)fstar=tstar
      imag0=imag

! datos a pantalla
!
      write(*,*)imag,' ',hw
      tstar=0
    endif

! controla filtro y Savart
!
    if(fil0 /= fil)then
      write(*,*)'Wrong filter (it must be ',fil0,'):'
      write(*,*)'Image: ',imag,' ID: ',id,' Filter: ',fil
      close(7)
      stop
    endif

    if(sav0 /= sav)then
      write(*,*)'Different Savart position (it must be ',sav0,'):'
      write(*,*)'Image: ',imag,' ID: ',id,' Savart Pos.: ',sav,' deg'
      close(7)
      stop
    endif

! numero de estrella y posicion HW
!
    tstar=tstar+1
    ii=int(hw/22.5_d+0.5)+1
    phi(ii)=hw/180._d*pi

! flujo total y area del cielo
!
    fso(tstar,ii)=fso(tstar,ii)+vv(1)*vv(2)
    aso(tstar,ii)=aso(tstar,ii)+vv(2)

! radios de aperturas (los lee una sola vez)
!
    if(imag0 == imag .and. (tstar == 1))then
      do ij=1,aper
        rap(ij)=vv(2+ij)
      enddo
    endif

! flujo ordinario
!
    do ij=1,aper
      fxo(tstar,ii,ij)=fxo(tstar,ii,ij)+vv(aper+2+ij)
    enddo

! area apertura 
!
    do ij=1,aper
      aro(tstar,ii,ij)=aro(tstar,ii,ij)+vv(2*aper+2+ij)
    enddo
 

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! lee datos de imagen extraordinaria
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    read(7,*)imag,id,(vv(jj),jj=1,2+3*aper),fil,hw,sav

! flujo total y area del cielo
!
    fse(tstar,ii)=fse(tstar,ii)+vv(1)*vv(2)
    ase(tstar,ii)=ase(tstar,ii)+vv(2)

! flujo extraordinario
!
    do ij=1,aper
      fxe(tstar,ii,ij)=fxe(tstar,ii,ij)+vv(aper+2+ij)
    enddo

! area apertura
!
    do ij=1,aper
      are(tstar,ii,ij)=are(tstar,ii,ij)+vv(2*aper+2+ij)
    enddo

  enddo
  close(7)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! comienza reduccion para cada objeto
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  open(8,file='polred.out')
  write(8,'(a)')'# POLRED v1.2 - Reduction program for CASPOL'
  write(8,'(a)')'#'
  write(8,'(a,a,a,a)')'# From: ',fecha0,' UT: ',hora0
  write(8,'(a,a,a,a)')'# To  : ',fecha,' UT: ',hora
  write(8,'(a)')'#'
  write(8,'(a,2f9.3)')'# Instr. Q (%)   : ',vq0*100._d,sq0*100._d
  write(8,'(a,2f9.3)')'# Instr. U (%)   : ',vu0*100._d,su0*100._d
  write(8,'(a,f8.3)')'# Instr. TH (deg): ',th0
  write(8,'(a)')'#'
  if(ren == 0)then
    write(8,'(a)')'# No renormalization'
    write(8,'(a)')'#'
  else
    write(8,'(a)')'#'
    write(8,'(a)')'#'
  endif

! abre archivo para guardar angulos e intensidades
!
  open(9,file='polred.cur')

  do ij=1,fstar
    write(8,'(/a,i4/)')'# Star :',ij
    write(8,'(a,a)')' Aper    Q%      eQ%     U%      eU%    Pol%    ePol%', &
      '   TH     eTH    eTeo%'

! indica el numero de estrella en polred.cur
!
    write(9,*)'# star: ',ij

! reduce cada apertura
!
    do jj=1,aper

! indica la apertura en polred.cur
!
      write(9,*)'# Aper: ',rap(jj)

! pone a cero los contadores
!
      vcos=0._d
      vsin=0._d
      tfxo=1._d
      tfxe=1._d
      tfx=0._d
      tfs=0._d
      arap=0._d
      arsk=0._d
      rdn2=0._d
      ihw=0

! calcula flujos netos, flujos totales y areas totales 
! para objeto y cielo
!
      do ii=1,16

! hay cuentas en esta posicion de la lamina?
!
        if(phi(ii) >= 0._d)then
          ihw=ihw+1
          vcos(ii)=cos(4._d*phi(ii))
          vsin(ii)=sin(4._d*phi(ii))

! flujos netos (objeto - cielo)
!
          fxo(ij,ii,jj)=fxo(ij,ii,jj)-aro(ij,ii,jj)*fso(ij,ii)/aso(ij,ii)
          fxe(ij,ii,jj)=fxe(ij,ii,jj)-are(ij,ii,jj)*fse(ij,ii)/ase(ij,ii)

! flujos totales ordinario, extraordinario, del objeto y del cielo
!
          tfxo=tfxo+fxo(ij,ii,jj)
          tfxe=tfxe+fxe(ij,ii,jj)
          tfx=tfx+(fxo(ij,ii,jj)+fxe(ij,ii,jj))/2._d
          tfs=tfs+(aro(ij,ii,jj)*fso(ij,ii)/aso(ij,ii)+are(ij,ii,jj)* &
            fse(ij,ii)/ase(ij,ii))/2._d

! area total para el objeto y cielo
!
          arap=arap+(aro(ij,ii,jj)+are(ij,ii,jj))/2._d
          arsk=arsk+(aso(ij,ii)+ase(ij,ii))/2._d

! contribucion por readout noise
!
!          rdn2=rdn2+rdnoise**2*(aro(ij,ii,jj)+are(ij,ii,jj))/2._d
        endif
      enddo

!contribucion por readout noise
!
      rdn2=rdnoise**2*arap/float(ihw)

! error teorico
!
      tfx=tfx*gain/float(ihw)
      tfs=tfs*gain/float(ihw)
      snt=sqrt(tfx+(1._d+1._d/arsk)*(tfs+rdn2))/tfx

! factor de renormalizacion
!
!      scl=(tfxo/tfxe)**(1._d/float(ihw))
      scl=tfxe/tfxo
      if(ren == 0)scl=1._d

! calcula intensidad y el error
!
      vint=0._d

      do ii=1,16

! hay cuentas en esta posicion de la lamina?
!
        if(phi(ii) >= 0._d)then
          vint(ii)=(fxe(ij,ii,jj)-fxo(ij,ii,jj)*scl)/(fxe(ij,ii,jj)+fxo(ij,ii,jj)*scl)
          sigz=2._d*scl/(fxe(ij,ii,jj)+fxo(ij,ii,jj)*scl)**2*sqrt(fxe(ij,ii,jj)* &
              fxo(ij,ii,jj)**2+fxo(ij,ii,jj)*fxe(ij,ii,jj)**2)

! guarda phi(i), z(i) y su error
!
          write(9,*)phi(ii)*180._d/pi,vint(ii)*100._d,sigz*100._d
        endif
      enddo

! calcula sumas para encontrar la polarizacion
!
      vcc=0._d
      vss=0._d
      vcs=0._d
      vcz=0._d
      vsz=0._d

      do ii=1,16

! hay cuentas en esta posicion de la lamina?
!
        if(phi(ii) >= 0._d)then
          vcc=vcc+vcos(ii)**2
          vss=vss+vsin(ii)**2
          vcs=vcs+vcos(ii)*vsin(ii)
          vcz=vcz+vcos(ii)*vint(ii)
          vsz=vsz+vsin(ii)*vint(ii)
        endif
      enddo

! calcula determinante del sistema
!
      det=vcc*vss-vcs**2
      sq=det/vcc
      su=det/vss

! calcula componentes
!
      vq=(vss*vcz-vsz*vcs)/det
      vu=(vcc*vsz-vcz*vcs)/det

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! polarizacion observada
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      pol=sqrt(vq**2+vu**2)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! calculo de los errores
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      suma2=0._d

      do ii=1,16

! hay cuentas en esta posicion de la lamina?
!
        if(phi(ii) >= 0._d)then
          suma2=suma2+(vq*vcos(ii)+vu*vsin(ii)-vint(ii))**2
        endif
      enddo

! errores
!
      sig=sqrt(suma2/(float(ihw)-2._d))
      sigq=sig/sqrt(sq)
      sigu=sig/sqrt(su)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! polarizacion corregida
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      vq=vq-vq0
      vu=vu-vu0
      pol=sqrt(vq**2+vu**2)
      sigq=sqrt(sigq**2+sq0**2)
      sigu=sqrt(sigu**2+su0**2)

      if(pol > 0._d)then
        sigpol=sqrt(vq**2*(sigq/pol)**2+vu**2*(sigu/pol)**2)
      else
        sigpol=sqrt(sigq**2+sigu**2)
      endif
      sigth=atan(sigpol/pol)/2._d*180._d/pi

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! angulo de polarizacion y componentes finales
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      th=0.5_d*atan2(vu,vq)*180._d/pi+th0
      if(th > 180._d)th=th-180._d
      if(th < 0._d)th=th+180._d

      vq=pol*cos(2._d*th/180._d*pi)
      vu=pol*sin(2._d*th/180._d*pi)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! salida final de datos a archivo polred.out
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      vq=vq*100._d
      vu=vu*100._d
      sigq=sigq*100._d
      sigu=sigu*100._d
      pol=pol*100._d
      sigpol=sigpol*100._d
      snt=snt*100._d
      write(8,'(f5.1,6f8.3,2f7.2,2f8.3)')rap(jj),vq,sigq,vu,sigu,pol,sigpol,th,sigth,snt

    enddo
  enddo
  close(8)
      close(9)
  
  write(*,*)'HW positions: ',ihw
  write(*,*)'Stars: ',fstar

end program polred
